import { Component, OnInit } from '@angular/core';
import { Observable } from "rxjs";
import { Client } from '../models/Client';
import { ClientsService } from '../clients.service';



declare var $: any;

@Component({
  selector: 'app-clients-panel',
  templateUrl: './clients-panel.component.html',
  styleUrls: ['./clients-panel.component.css']
})

export class ClientsPanelComponent implements OnInit {
  appId = '';
  appSecret = '';
  tiposCliente = ['saas', 'hosting', 'local' ];
  customers: Observable<Client[]>;
  urlValidator: RegExp;
  tokenValidator : RegExp;
  catchableToken: any;
  fieldAux: string;
  currentField: any;
  currentBtn: any;
  catchableClient: any;
  constructor( private clientsService: ClientsService ) {
    
    this.urlValidator =  /https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_\+.~#?&//=]*)/;
    this.tokenValidator = /^([A-Za-z0-9_]{8})\-{1}?([A-Za-z0-9_]{4})\-{1}?([A-Za-z0-9_]{4})\-{1}?([A-Za-z0-9_]{4})\-{1}?([A-Za-z0-9_]{12})$/;
    this.currentField = "";
    this.currentBtn = "";
    this.catchableToken = null;
    this.catchableClient = null;
  }

  ngOnInit() {
    this.reloadData();
  }

  reloadData() {
    this.reloadAnimation();
    this.customers = this.clientsService.getCustomers();
    this.clientsService.getParams().subscribe(resultado =>{ 
      this.appSecret = resultado.data.CLIENTSECRET;
      this.appId = resultado.data.CLIENTID;
    });
    //this.params = this.clientsService.getParams().data;
   }

  saveParams(){
  let data = {
    appId : this.appId,
    appSecret : this.appSecret,
  };
  this.clientsService.updateParams(data).subscribe(res => res.success ? this.reloadData() : this.showalert(res.message, 'danger'));
  $("#addNewParams").modal('hide');
  }
  
   addClient( name, url, tipoCliente, usuarioDb ) {
    if( name.value ) {
      if( this.urlValidator.test(url.value) && url.value !== '' ) {
        let data = {
          nomeCliente : name.value, 
          urlCliente  : url.value,
          tipoCliente : tipoCliente.value,
          usuarioDb   : usuarioDb.value, 
        } 
        console.log(data);
        
        this.clientsService.addCustomer(data).subscribe(res => res.success ? this.reloadData() : this.showalert(res.message, 'danger'));
        this.resetFields([name,url]);
        $("#addNewClientModal").modal('hide');
      }
    } else {
      this.invalidField(name);
    }
  }

  
  urlVerify( url: string, field ) {
    if( url ) {
      if( !this.urlValidator.test(url) ) {
        this.removeValidadeInfo(field);
        this.invalidField(field);
       }else{
        this.removeValidadeInfo(field);
        this.validField(field);
       }
    }else{
      this.removeValidadeInfo(field);
    }
  }

  validField( field ) {
    field.classList.add("is-valid")
  }

  invalidField( field ) {
    field.classList.add("is-invalid")
  }

  removeValidadeInfo( field ) {
    if(field.classList.contains("is-invalid")) {field.classList.remove("is-invalid");}
    if(field.classList.contains("is-valid")) {field.classList.remove("is-valid");}
  }

  resetFields ( fields:Array<any> ) {
    for (const field of fields) {
      field.value = "";
      this.removeValidadeInfo(field)
    }
  }

  unlockField( field ) {
    field.disabled = false;
  }

  lockField( field ) {
    field.disabled = true;
  }

  editToggle( field, info, editBtn ) {
    let saveBtnIcon = "save";
    let editBtnIcon = "create";

   if(field.value !== "") {
      if(field.disabled) {
        if(field === this.currentField || this.currentField === "") { 
          this.currentField = field;
          this.currentBtn = editBtn
          this.fieldAux = field.value;
          this.unlockField(field);
          editBtn.innerHTML = saveBtnIcon;
        } else {
          this.lockField(this.currentField);
          this.removeValidadeInfo(this.currentField);
          this.currentBtn.innerHTML = editBtnIcon;

          this.currentBtn = editBtn;
          this.fieldAux = field.value;
          this.currentField = field;
          this.unlockField(field);
          editBtn.innerHTML = saveBtnIcon;
        }
      } else {
        if(field.value === this.fieldAux) {
          this.fieldAux = "";

        } else {
          if( info.idCliente ) {
            this.editClient(info);
          } else {
            this.editToken(info);
          }
        }
        this.lockField(field);
        this.removeValidadeInfo(field);
        editBtn.innerHTML = editBtnIcon;
      }
    } else { 
      this.showalert("Alteração não pode ser concluída! O campo deve estar devidamente preenchido.", 'danger');
      this.lockField(field)
      editBtn.innerHTML = editBtnIcon;
      this.reloadData();
    }
  }

  addNewToken( client, token ) {
    //if( this.tokenValidator.test( token ) && token.length == 36 ) { //validation to ubereats
    if (token.length != 0) {
        let data = { idCliente: client.idCliente,token: token };
        this.clientsService.addNewToken(data).subscribe(res => res.success ? this.reloadData() : this.showalert(res.message, 'danger'));
    } else {
      this.showalert('Token inválido', 'danger');
    }  
  }

  deleteToken() {
    if (this.catchableToken) {
      let data = {idToken : this.catchableToken};
      this.clientsService.deleteToken(data).subscribe(res => res.success ? this.reloadData() : this.showalert(res.message, 'danger'));
      //unset the catchableToken
      this.catchableToken = null;
    }
  }
  editClient( customer ) {
    if( this.urlValidator.test(customer.url) && customer.url !== '' ){
      let data = {
        idCliente : customer.idCliente,
        nomeCliente : customer.cliente,
        urlCliente: customer.url,
        tipoCliente: customer.tipoCliente,
        usuarioDb: customer.usuarioDb

      }
      console.log(data);
      
      this.clientsService.editClientInfo(data).subscribe(res => res.success ? this.reloadData() : this.showalert(res.message, 'danger'));
    }else{
      this.reloadData();
      this.showalert('Url inválida','danger');
    }
  }

  editToken( token ) {
    let data = {idToken: token.idToken, token: token.token};
    console.log(data);
    this.clientsService.editToken(data).subscribe(res => res.success ? this.reloadData() : this.showalert(res.message, 'danger'));
    
  }

  deleteClient() {
    if (this.catchableClient) {
      let data = {
        idCliente : this.catchableClient
      };
      this.clientsService.deleteClient(data).subscribe(res => res.success ? this.reloadData() : this.showalert(res.message, 'danger'));
      //unset the catchableToken
      this.catchableToken = null;
    }
  }

  showalert( message, alerttype ) {
    $('#alert_placeholder').append(`
    <div id="alertdiv" class="alert alert-${alerttype} alert-dismissible fade show" role="alert">
      Erro: <strong>${message}</strong>
      <button type="button" class="close" data-dismiss="alert" aria-label="Close">
        <span aria-hidden="true">&times;</span>
      </button>
    </div>`);
    setTimeout( function() { // this will automatically close the alert and remove this if the users doesnt close it in 10 secs
      $("#alertdiv").remove();
    }, 10000 );
  }

  tokenInputWatch( field ) {
    if ( this.tokenValidator.test(field.value) ) {
      if ( field.value.length == 36 ) {
        this.removeValidadeInfo(field);
        this.validField(field);
        
      } else {
        this.removeValidadeInfo(field);
        this.invalidField(field);
      }

    } else if ( field.value != '' ) {
      this.removeValidadeInfo(field);
      this.invalidField(field);  
    } else {
      this.removeValidadeInfo(field);
    } 
  }
  reloadAnimation() {
    let overlay = document.getElementById("overlay");
    overlay.style.display = overlay.style.display = 'block';
    setTimeout(() => {
      overlay.style.display = overlay.style.display = 'none'; 
    },650);
  
  }
  catchClient( client ) {
   this.catchableClient = client;
  }
  catchToken( token ) {
    this.catchableToken = token;
  }

  handleInputDb( file ) {
    console.log(file);
    let myInputLabel  = document.getElementById("label-inputDb");
    let labelInfo = file.value.substr(11)
    myInputLabel.innerHTML = labelInfo;
  }
}
