<?php

namespace App\Util;

use App\Exception;
use App\Util\Telegram;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

class Util {
    
    const TELEGRAM_BOT_TOKEN = "5763348046:AAFM5Np9ckWaWRsGSu0wV6VLnN4gRFjTXlI";
    
    
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    } 
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }
    
    public function logError ($path, $content, $branch, $orderNumber) {
        $folder = $path  . "/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
        $fileName = $branch . "-" . $orderNumber.".txt";
        $currentFile = $folder . "/" . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
        }
        
        $line = date("d-m-Y H:i:s") . " - " . $content . PHP_EOL;
        file_put_contents($currentFile, $line, FILE_APPEND);
    }
    
    public function logOrder ($path, $content, $branch) {
        $contentT = json_decode($content, true);
        
        $folder = $path  . "/" . date("Y") . "/" . date("m") . "/" . date("d");
        $fileName = $branch."-".$contentT["displayId"]."-".round(microtime(true) * 1000).".json";
        $currentFile = $folder . "/" . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
        }
        
        file_put_contents($currentFile, $content);
        
        return $currentFile;
    }
    
    public function sendMail($recipient, $cc, $subject, $body, $params, $currentFile)
    {
        return 0;
        try {
            $configsFile = $this->getConfigsFile();
        
            $mail = new PHPMailer(true);

            $mail->isSMTP();                            
            $mail->Host       = $configsFile["host"];          
            $mail->SMTPAuth   = true;                   
            $mail->Username   = $configsFile["username"];
            $mail->Password   = $this->decrypt($configsFile["password"]); 
            $mail->SMTPSecure = $configsFile["SMTPSecure"];    
            $mail->Port       = $configsFile["port"];      
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";
            
            $mail->setFrom($configsFile["from"], "Teknisa Retail");

            if(empty($cc)) {
                $cc = ["suporte.odhen@teknisa.com"];
            }
            
            foreach ($recipient as $r) {
                $mail->addAddress($r);
            }
            
            foreach ($cc as $c) {
                $mail->addCC($c);
            }
            
            $mail->isHTML(true);                                  
            $mail->Subject = $subject;
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
            
            // TELEGRAM MESSAGE
            
            $messageBody = "Pedido: {$params[':NUMERO_PEDIDO']} -- Cliente: {$configsFile['client']} -- Servidor: {$configsFile['server']} \r\n\n{$params[':ERRO_MSG']} \r\n\nJSON: {$currentFile} \r\n\nGerado em: ". date("d/m/Y H:i:s");
            
            $messageBody = str_replace(['<br>', '    ', '<b>', '</b>'], '', $messageBody);
            
            if(!empty($configsFile["telegramChannel"])) {
                $this->sendTelegramMessage($messageBody);
            }
            
        } catch (Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    }

    public function getPendingOrdersFile() {
        $configsFile = $this->getConfigsFile();

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $configsFile['logPath'] . "/$clientFolder";

        $folder = $basePath."/dispatch";

        $fileName = "/pendingOrders.json";
        $currentFile = $folder . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
            file_put_contents($currentFile, json_encode([]));
        }

        return $currentFile;
    }
    
    public function getCurrentTimezone($state) {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }
    
    private function decrypt($string) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
    
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
    
        $key = hash('sha256', $secret_key);
        $iv  = substr(hash('sha256', $secret_iv), 0, 16);

        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        return $output;
    } 

    public function getNetworkVerbosePath() {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $basePath     = $config['logPath'] . "/$clientFolder";

        $folder      = $basePath . "/networkVerbose/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
        $timeStamp = date("H-i-s");
        $fileName = "$timeStamp-verbose.txt";
        $currentFile = "$folder/$fileName";
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
        }
        return $currentFile;
    }

    public function writeLog($type, $fileName, $content) 
    {

        $configsFile = $this->getConfigsFile();

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $basePath     = $configsFile['logPath'] . "/$clientFolder";
        
        if($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }

        if($type == 'network_verbose') {
            $folder      = $basePath . "/networkVerbose/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }

        if($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
            $currentFile = "$folder" . date("d") . ".txt";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }

        if($type == 'events') {
            $folder      = $basePath . "/events/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

	        file_put_contents($currentFile, $content, FILE_APPEND);
        }

        if($type == 'cancellation') {
            $folder      = $basePath . "/cancellation/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

	        file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'combinedProdError') {
            $folder      = $basePath . "/combinedProdError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }
    }

    public function getMailList() {
        $list = [];

        $sql = "
            SELECT DSEMAILTEC 
              FROM LISTAEMAILTEC
             WHERE CDPROJETO = '003'
        ";
        
        $result = $this->connection->fetchAll($sql);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("
                Não existe um e-mail cadastrado para receber os erros da integração.<br>
                Para cadastrar um e-mail, verifique a seção 'Parametrização da Integração', no manual da integração.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("
                Não existe um e-mail válido cadastrado para receber os erros da integração.<br>
                Verifique a seção 'Parametrização da Integração', no manual da integração.");
        }
        
        return $list;
    }
    
    public function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
    public function sendTelegramMessage($body)
    {

        $telegram = new Telegram();
        
        $configsFile = $this->getConfigsFile();
        
        $telegram->setCredentials('bot'.self::TELEGRAM_BOT_TOKEN, $configsFile["telegramChannel"]);
        
        return $telegram->msg($body);
    }
    
    public function getConfigsFile()
    {
        global $CONFIG_FILE;

        $config_file = $CONFIG_FILE;
        return json_decode($config_file, true);
    }


    private function getValueByProperty($object, $prop, $default = '') {
        if(isset($object[$prop]) && !empty($object[$prop])) {
            return $object[$prop];
        }
        return $default;
    }

    public function getRequestIp() {
        $ip = $this->getValueByProperty($_SERVER,  'HTTP_CLIENT_IP', '0|');
        $ip .= $this->getValueByProperty($_SERVER, 'HTTP_X_FORWARDED_FOR', '0|');
        $ip .= $this->getValueByProperty($_SERVER, 'REMOTE_ADDR', '0|');
        
        return $ip;
    }

}
